<?php
namespace Drupal\commerce_bpc_bpc;
use Drupal;
use Drupal\Core\Url;
use Exception;
require_once DRUPAL_ROOT ."/". \Drupal::service('extension.list.module')->getPath('commerce_bpc_bpc') ."/src/cfg/include.php";
/**
* Sends requests to the server encoding request parameters and decoding
* results with given encoder and decoder. returns error if a timeout occurs
*/
class CommerceBPCBPCApi
{
const maxFileReadingStringLength = 4096;
const internalErrorCode = 10000;
const orderStatusPending = 0;
const orderStatusPreHold = 1;
const orderStatusAuthorized = 2;
const orderStatusReversed = 3;
const orderStatusPartlyRefunded = 4;
const orderStatusAuthACS = 5;
const orderStatusDeclined = 6;
public $cms_name = "Drupal";
public $cms_version = Drupal::VERSION;
public $module_version = '2.1.3';
public $language = BPC_BPC_DEFAULT_LANGUAGE;
private $serverUrl;
private $timeout;
private $userName;
private $password;
private $isDoubleStaged;
public $enable_cacert;
public $cacert_path = null;
private $isTestMode;
private $orderBundle;
private $taxSystem;
private $clientId;
private $backToShopUrl;
private $backToShopUrlName;
private $billingPayerData;
private $order_currency;
private $isLogging;
public function __construct($params = array())
{
foreach ($params as $key => $value) {
$this->{$key} = $value;
}
}
function registerOrder($payment_id, $amount, $return_url, $fail_url = null, $description = "")
{
$jsonParams = array(
'CMS:' => $this->cms_name . " " . $this->cms_version . ' + Commerce',
'Module-Version: ' => $this->module_version
);
if ( !empty($this->backToShopUrl)
) {
$jsonParams['backToShopUrl'] = $this->backToShopUrl;
}
$args = array(
'orderNumber' => $payment_id . "_" . time(),
'amount' => $amount,
'returnUrl' => $return_url,
'language' => $this->language,
'jsonParams' => json_encode($jsonParams),
);
if (!empty($this->clientId) && $this->clientId) {
$args['clientId'] = $this->clientId;
}
if (!empty($this->billingPayerData)) {
$args['billingPayerData'] = json_encode($this->billingPayerData);
}
if (!empty($this->order_currency)) {
$args['currency'] = $this->order_currency;
}
if (!empty($this->orderBundle)) {
$args['orderBundle'] = json_encode($this->orderBundle);
$args['taxSystem'] = $this->taxSystem;
}
if (!empty($description)) {
$args['description'] = $description;
}
if (!empty($fail_url) && $fail_url != $return_url) {
$args['failUrl'] = $fail_url;
}
$method = $this->isDoubleStaged ? 'registerPreAuth.do' : 'register.do';
return $this->callMethod($method, $args);
}
/**
* Sends data compressed by deflate algorithm to the server.
*
* @param string $method_url_suffix
*  Url suffix.
* @param array $args
*  Data to be send to server.
*
* @return array
*  response.
*/
protected function callMethod($method_url_suffix, array $args = array())
{
$test_url = BPC_BPC_TEST_URL;
$prod_url = BPC_BPC_PROD_URL;
if (defined('BPC_BPC_PROD_URL_ALTERNATIVE_DOMAIN') && defined('BPC_BPC_PROD_URL_ALT_PREFIX')) {
if (substr($this->userName, 0, strlen(BPC_BPC_PROD_URL_ALT_PREFIX)) == BPC_BPC_PROD_URL_ALT_PREFIX) {
$pattern = '/^https:\/\/[^\/]+/';
$prod_url = preg_replace($pattern, rtrim(BPC_BPC_PROD_URL_ALTERNATIVE_DOMAIN, '/'), $prod_url);
} else {
}
}
if (file_exists(DRUPAL_ROOT ."/". \Drupal::service('extension.list.module')->getPath('commerce_bpc_bpc') ."/src/cfg/cacert.cer")) {
if ($this->enable_cacert === true) {
$this->cacer_path = DRUPAL_ROOT ."/". \Drupal::service('extension.list.module')->getPath('commerce_bpc_bpc') ."/src/cfg/cacert.cer";
}
}
$action_adr = $this->isTestMode ? $test_url : $prod_url;
try {
$action_adr .= $method_url_suffix;
$args['userName'] = $this->userName;
$args['password'] = $this->password;
$curl_opt = array(
CURLOPT_HTTPHEADER => array(
'CMS:' => $this->cms_name . " " . $this->cms_version . ' + Commerce',
'Module-Version: ' => $this->module_version
),
CURLOPT_VERBOSE => true,
CURLOPT_SSL_VERIFYHOST => false,
CURLOPT_URL => $action_adr,
CURLOPT_RETURNTRANSFER => true,
CURLOPT_POST => true,
CURLOPT_POSTFIELDS => http_build_query($args, '', '&'),
);
$ssl_verify_peer = false;
if (isset($this->cacer_path)) {
$ssl_verify_peer = true;
$curl_opt[CURLOPT_CAINFO] = $this->cacer_path;
}
$curl_opt[CURLOPT_SSL_VERIFYPEER] = $ssl_verify_peer;
$ch = curl_init();
curl_setopt_array($ch, $curl_opt);
$response = curl_exec($ch);
curl_close($ch);
$communication_start_time = time();
if (empty($response)) {
$this->throwConnectionException($communication_start_time, t("Problem occurred while reading data from bank's server."));
}
$result = json_decode($response, TRUE);
if ($this->isLogging) {
if (isset($result['errorCode']) && $result['errorCode']) {
\Drupal::logger('commerce_bpc_bpc')->warning('API "%payment_method" call, ErrorCode = "%error_code", message = "%message", <br/> post: %post<br/> response: %response', array(
'%payment_method' => $action_adr,
'%error_code' => isset($result['errorCode']) ? $result['errorCode'] : '',
'%message' => isset($result['errorMessage']) ? $result['errorMessage'] : '',
'%post' => htmlentities(print_r($args, TRUE)),
'%response' => htmlentities($response),
));
} else {
\Drupal::logger('commerce_bpc_bpc')->debug('API "%payment_method" call, ErrorCode = "%error_code", message = "%message", <br/> post: %post<br/> response: %response', array(
'%payment_method' => $action_adr,
'%error_code' => isset($result['errorCode']) ? $result['errorCode'] : '',
'%message' => isset($result['errorMessage']) ? $result['errorMessage'] : '',
'%post' => htmlentities(print_r($args, TRUE)),
'%response' => htmlentities($response),
));
}
}
return $result;
} catch (Exception $e) {
$errorCode = isset($result['errorCode']) ? $result['errorCode'] : '';
if ($errorCode) {
\Drupal::logger('commerce_bpc_bpc')->warning('API "%payment_method" call, ErrorCode = "%error_code", message = "%message", <br/> post: %post', array(
'%payment_method' => $action_adr,
'%error_code' => $errorCode,
'%message' => $e->getMessage(),
'%post' => htmlentities(print_r($args, TRUE)),
));
} else {
\Drupal::logger('commerce_bpc_bpc')->debug('API "%payment_method" call, ErrorCode = "%error_code", message = "%message", <br/> post: %post', array(
'%payment_method' => $action_adr,
'%error_code' => $errorCode,
'%message' => $e->getMessage(),
'%post' => htmlentities(print_r($args, TRUE)),
));
}
return array(
'errorCode' => self::internalErrorCode,
'errorMessage' => $e->getMessage(),
);
}
}
/**
* @param int $communication_start_time
* @param string $message
*
* @throws Exception
*/
private function throwConnectionException($communication_start_time, $message)
{
$timeout_msg = $this->isTimedOut($communication_start_time) ? t('Bank server response time exceeded (@timeout)', array('@timeout' => $this->timeout)) : '';
throw new Exception($message . ' ' . $timeout_msg);
}
/**
* @param int $communication_start_time
*
* @return bool
*/
private function isTimedOut($communication_start_time)
{
return (time() - $communication_start_time) >= $this->timeout;
}
/**
* @param string $gateway_order_id
* @param int $amount
*
* @return mixed
*/
public function confirmOrderPayment($gateway_order_id, $amount = 0)
{
return $this->callMethod('deposit.do', array(
'orderId' => $gateway_order_id,
'amount' => $amount,
));
}
/**
* @param int $order_id
*
* @return array
*/
public function getOrderStatusByOrderId($order_id)
{
return $this->callMethod('getOrderStatusExtended.do', array('orderNumber' => $order_id));
}
/**
* @param string $gateway_order_id
*
* @return mixed
*/
public function getOrderStatusByGatewayOrderId($gateway_order_id)
{
return $this->callMethod('getOrderStatusExtended.do', array('orderId' => $gateway_order_id));
}
/**
* @param string $gateway_order_id
*
* @return mixed
*/
public function reverseOrderPayment($gateway_order_id)
{
return $this->callMethod('reverse.do', array('orderId' => $gateway_order_id));
}
/**
* @param string $gateway_order_id
* @param int $amount
*
* @return mixed
*/
public function refundOrderPayment($gateway_order_id, $amount)
{
return $this->callMethod('refund.do', array(
'orderId' => $gateway_order_id,
'amount' => $amount
));
}
/**
* @param string $date
* @param array $transaction_states
* @param bool $search_by_created_date
* @param int $num_entries
*
* @return array
*/
public function getOperationsList($date, $transaction_states = array(), $search_by_created_date = TRUE, $num_entries = 100)
{
if (!$transaction_states) {
$transaction_states = array('CREATED', 'APPROVED', 'DEPOSITED', 'DECLINED', 'REVERSED', 'REFUNDED');
}
$params = array(
'from' => "{$date}000000",
'to' => "{$date}235959",
'size' => $num_entries,
'transactionStates' => implode(', ', $transaction_states),
'searchByCreatedDate' => $search_by_created_date,
'merchants' => '',
);
return $this->callMethod('getLastOrdersForMerchants.do', $params);
}
public function _updateGateSettings($params)
{
$login = $this->userName;
$password = $this->password;
if ($this->isTestMode) {
$action_adr = BPC_BPC_TEST_URL;
$gate_url = str_replace("payment/rest", "mportal/mvc/public/merchant/update", $action_adr);
} else {
$action_adr = BPC_BPC_PROD_URL;
$gate_url = str_replace("payment/rest", "mportal/mvc/public/merchant/update", $action_adr);
if (defined('BPC_BPC_PROD_URL_ALTERNATIVE_DOMAIN')) {
$pattern = '/^https:\/\/[^\/]+/';
$gate_url = preg_replace($pattern, rtrim(BPC_BPC_PROD_URL_ALTERNATIVE_DOMAIN, '/'), $gate_url);
}
}
$gate_url .= substr($login, 0, -4); // we guess username = login w/o "-api"
$headers = array(
'Content-Type:application/json',
'Authorization: Basic ' . base64_encode($login . ":" . $password)
);
$data['callbacks_enabled'] = true;
$data['callback_http_method'] = "GET";
$data['callback_type'] = 'STATIC';
$data['callback_operations'] = "deposited,approved,declinedByTimeout";
$data['callback_addresses'] = $params['callback_addresses'] . "?callback=true";
$ch = curl_init();
curl_setopt_array($ch, array(
CURLOPT_HTTPHEADER => $headers,
CURLOPT_VERBOSE => true,
CURLOPT_SSL_VERIFYHOST => false,
CURLOPT_SSL_VERIFYPEER => false,
CURLOPT_URL => $gate_url,
CURLOPT_RETURNTRANSFER => true,
CURLOPT_POST => true,
CURLOPT_POSTFIELDS => json_encode($data),
CURLOPT_ENCODING => "gzip",
));
$response = curl_exec($ch);
if ($this->isLogging) {
\Drupal::logger('commerce_bpc_bpc')->debug('Set callback_addresses, message = "%message", <br/>URL: %url<br/>POST: %post', array(
'%message' => $response,
'%url' => $gate_url,
'%post' => json_encode($data),
));
}
}
}
