<?php
namespace Drupal\commerce_bpc_bpc\PluginForm\OffsiteRedirect;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\commerce_price\Calculator;
use Drupal\commerce_price\Entity\Currency;
use Drupal\commerce_price\Price;
use Drupal\commerce_bpc_bpc\CommerceBPCBPCApi;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_order\Entity\Order;
use Drupal\commerce_order\Entity\OrderItem;
require_once DRUPAL_ROOT . "/" . \Drupal::service('extension.list.module')->getPath('commerce_bpc_bpc') . "/src/cfg/include.php";
class PaymentOffsiteForm extends BasePaymentOffsiteForm
{
/**
* {@inheritdoc}
*/
public function buildConfigurationForm(array $form, FormStateInterface $form_state)
{
$form = parent::buildConfigurationForm($form, $form_state);
/** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
$payment = $this->entity;
/** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
$payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
$data = [];
$payment_gateway_configuration = $payment_gateway_plugin->getConfiguration();
if (!empty($payment_gateway_configuration['token'])) {
$decoded_credentials = base64_decode($payment_gateway_configuration['token']);
list($l, $p) = explode(':', $decoded_credentials);
$params['userName'] = $l;
$params['password'] = $p;
} else {
$params['userName'] = $payment_gateway_configuration['username'];
$params['password'] = $payment_gateway_configuration['password'];
}
$params['isDoubleStaged'] = $payment_gateway_configuration['double_staged'] == 0 ? false : true;
$params['isTestMode'] = $payment_gateway_configuration['mode'] == 'live' ? false : true;
$params['enable_cacert'] = $payment_gateway_configuration['enable_cacert'] == 0 ? false : true;
$params['isLogging'] = $payment_gateway_configuration['logging'] == 0 ? false : true;
$params['isSendOrder'] = $payment_gateway_configuration['send_order'] == 0 ? false : true;
$params['FFDVersion'] = $payment_gateway_configuration['FFDVersion'];
$params['taxSystem'] = $payment_gateway_configuration['tax_system'];
$order = $payment->getOrder();
$amount = $order->getTotalPrice();
$current_user = \Drupal::currentUser();
$user_id = $current_user->id();
if ($user_id && $user_id > 0) {
$client_email = !empty($order->getEmail()) ? $order->getEmail() : "";
$params['clientId'] = md5($user_id . $client_email . \Drupal::request()->getSchemeAndHttpHost());
}
$params['backToShopUrl'] = $payment_gateway_configuration['backToShopUrl'];
if (defined('BPC_BPC_SEND_CLIENT_FULL_INFO') && BPC_BPC_SEND_CLIENT_FULL_INFO === true) {
$params['billingPayerData'] = $this->_getBillingPayerData($order);
}
if (defined('BPC_BPC_MANDATORY_CURRENCY') && BPC_BPC_MANDATORY_CURRENCY === true) {
$currency_code = $order->getTotalPrice()->getCurrencyCode();
$currency_storage = \Drupal::entityTypeManager()->getStorage('commerce_currency');
$currency = $currency_storage->load($currency_code);
if ($currency) {
$params['order_currency'] = $currency->getNumericCode();
}
}
if (defined('BPC_BPC_ENABLE_CART_OPTIONS') && BPC_BPC_ENABLE_CART_OPTIONS == true && $params['isSendOrder']) {
$dp_items = $order->getItems();
$items = array();
$itemsCnt = 1;
foreach ($dp_items as $key => $value) {
$itemPrice = round($value->getUnitPrice()->getNumber(), 2) * 100;
$tax_type = $this->getTaxType($value, $payment_gateway_configuration['tax_type']);
$item = array(
'positionId' => $itemsCnt++,
'itemAmount' => $value->getQuantity() * $itemPrice,
'name' => $value->getTitle(),
'itemPrice' => $itemPrice,
'quantity' => array(
'value' => $value->getQuantity(),
'measure' => ($params['FFDVersion'] == "v1_05") ? BPC_BPC_MEASUREMENT_NAME : BPC_BPC_MEASUREMENT_CODE
),
'tax' => array(
'taxType' => $tax_type
),
'itemCode' => $value->id()
);
$attributes = array();
$attributes[] = array(
"name" => "paymentMethod",
"value" => $payment_gateway_configuration['ffd_paymentMethodType']
);
$attributes[] = array(
"name" => "paymentObject",
"value" => $payment_gateway_configuration['ffd_paymentObjectType']
);
$item['itemAttributes']['attributes'] = $attributes;
$items[] = $item;
}
$order_bundle = array(
'orderCreationDate' => time(), //$order->changed,
'customerDetails' => array(
'email' => $order->getEmail()
),
'cartItems' => array('items' => $items)
);
$params['orderBundle'] = $order_bundle;
} //orderBundle
$gatewayAPI = new CommerceBPCBPCApi($params);
$payment->save();
if (defined('BPC_BPC_ENABLE_CALLBACK') && BPC_BPC_ENABLE_CALLBACK == true) {
$params['callback_addresses'] = $form["#return_url"];
$gatewayAPI->_updateGateSettings($params);
}
$response = $gatewayAPI->registerOrder(
$payment->id(),
$this->toMinorUnits($amount),
$form["#return_url"]
);
if (!isset($response['errorCode'])) {
$redirect_url = $response['formUrl'];
$payment->setRemoteId($response['orderId']);
$payment->save();
return $this->buildRedirectForm($form, $form_state, $redirect_url, $data, self::REDIRECT_GET);
} else {
\Drupal::messenger()->addMessage(t('Error # %code: %message', [ //PLUG-4598
'%code' => $response['errorCode'],
'%message' => $response['errorMessage']]), 'error');
}
}
/**
* Converts the given amount to its minor units.
*
* For example, 9.99 USD becomes 999.
*
* @param \Drupal\commerce_price\Price $amount
*   The amount.
*
* @return int
*   The amount in minor units, as an integer.
*/
protected function toMinorUnits(Price $amount)
{
/** @var \Drupal\commerce_price\Entity\CurrencyInterface $currency */
$currency = Currency::load($amount->getCurrencyCode());
$fraction_digits = $currency->getFractionDigits();
$number = $amount->getNumber();
if ($fraction_digits > 0) {
$number = Calculator::multiply($number, pow(10, $fraction_digits));
} else {
$number = $number * 100;
}
return round($number, 0);
}
/**
* @param string $currency_code
*
* @return string
*/
function getCurrencyNumericCode($currency_code)
{
$currency = Currency::load($currency_code);
return $currency->getNumericCode();
}
function getTaxType($value, $default_value)
{
$item_rate = -1; // wo VAT
foreach ($value->getAdjustments() as $k => $v) {
$item_rate = intval($v->getPercentage() * 100); // $tax_data['percentage']
}
if ($item_rate == 20) {
$tax_type = 6;
} else if ($item_rate == 18) {
$tax_type = 3;
} else if ($item_rate == 10) {
$tax_type = 2;
} else if ($item_rate == 0) {
$tax_type = 1;
} else {
$tax_type = $default_value;
}
return $tax_type;
}
function _getBillingPayerData($order) {
$billingPayerData = [];
$billingProfile = $order->getBillingProfile();
if ($billingProfile) {
/** @var \Drupal\address\AddressInterface $billingAddress */
$billingAddress = $billingProfile->get('address')->first();
if ($billingAddress) {
if (!empty($billingAddress->getLocality())) {
$billingPayerData['billingCity'] = $billingAddress->getLocality();
}
if (!empty($billingAddress->getCountryCode())) {
$billingPayerData['billingCountry'] = $billingAddress->getCountryCode();
}
if (!empty($billingAddress->getAddressLine1())) {
$billingPayerData['billingAddressLine1'] = $billingAddress->getAddressLine1();
}
if (!empty($billingAddress->getAddressLine2())) {
$billingPayerData['billingAddressLine2'] = $billingAddress->getAddressLine2();
}
if (!empty($billingAddress->getPostalCode())) {
$billingPayerData['billingPostalCode'] = $billingAddress->getPostalCode();
}
if (!empty($billingAddress->getAdministrativeArea())) {
$billingPayerData['billingState'] = $billingAddress->getAdministrativeArea();
}
}
}
return $billingPayerData;
}
}
